#include "CivBaseInpBuilder.h"

#include "CivInpBuilder.h"
#include "CivTypes.h"
#include "CivAssembly.h"

#define ASSERT(A,B)  if ((A) == nullptr || (B) == nullptr) return;

CivBaseInpBuilder::CivBaseInpBuilder() 
	:CivInpBuilder() mNewInp(new CivNewInp())
{
}

CivBaseInpBuilder::~CivBaseInpBuilder()
{
	delete mNewInp;
}

CivNewInp* CivBaseInpBuilder::getNewInp()
{
	return mNewInp;
}


CivBaseInpBuilder::CivBaseInpBuilder()
{

}

void CivBaseInpBuilder::buildNode()
{

	CivNode node;
	if (!mDbConn->getNode(node))
		return;

	mNewInp->setNode(node);
}

void CivBaseInpBuilder::buildPipe()
{
	ASSERT(mDbConn, mNewInp)

		CivPipe pipe;
	if (!mDbConn->getPipe(pipe))
		return;

	mNewInp->setPipes(pipe);
}

void CivBaseInpBuilder::buildTank()
{
	ASSERT(mDbConn, mNewInp)

		CivTank tank;
	if (!mDbConn->getTank(tank))
		return;

	mNewInp->setTank(tank);
}

void CivBaseInpBuilder::buildVavle()
{
	ASSERT(mDbConn, mNewInp)

		CivValve valve;
	if (!mDbConn->getValve(valve))
		return;

	mNewInp->setValves(valve);
}

void CivBaseInpBuilder::buildPump()
{
	ASSERT(mDbConn, mNewInp)

		CivPumps valve;
	if (!mDbConn->getPumps(valve))
		return;

	mNewInp->setPumps(valve);
}

void CivBaseInpBuilder::buildReservoir()
{
	ASSERT(mDbConn, mNewInp)

		CivReservoirs res;
	if (!mDbConn->getReservoirs(res))
		return;

	mNewInp->setReservoirs(res);
}

void CivBaseInpBuilder::buildEmitter()
{
	ASSERT(mDbConn, mNewInp)

		CivEmitters emmiters;
	if (!mDbConn->getEmitters(emmiters))
		return;

	mNewInp->setEmitters(emmiters);
}

void CivBaseInpBuilder::buildPatterns()
{
	ASSERT(mDbConn, mNewInp)

		CivPatterns pattern;
	if (!mDbConn->getPatterns(pattern))
		return;

	mNewInp->setPatterns(pattern);
}

void CivBaseInpBuilder::buildParamter()
{
	ASSERT(mDbConn, mNewInp)

		std::vector<CivParameter> parameters;
	if (!mDbConn->getParameter(parameters))
		return;

	size_t total = parameters.size();
	for (int i = 0; i < total; i++)
	{
		CivParameter param = parameters[i];
		mNewInp->setParamter(param, param.mType);
	}
}

void CivBaseInpBuilder::buildCoordnates()
{
	ASSERT(mDbConn, mNewInp)

		CivCoordinates coords;
	if (!mDbConn->getCoordinates(coords))
		return;

	mNewInp->setCoordnates(coords);
}

void CivBaseInpBuilder::buildVertices()
{
	return;
}

void CivBaseInpBuilder::buildLabels()
{
	ASSERT(mDbConn, mNewInp)

		CivLabels labels;
	if (!mDbConn->getLabels(labels))
		return;

	mNewInp->setLabels(labels);
}

void CivBaseInpBuilder::buildTags()
{
	ASSERT(mDbConn, mNewInp)

		CivTags tag;
	if (!mDbConn->getTags(tag))
		return;

	mNewInp->setTags(tag);
}

void CivBaseInpBuilder::buildBackDrop()
{
	return;
}

void CivBaseInpBuilder::buildQuality()
{
	ASSERT(mDbConn, mNewInp)


		CivQuality quality;
	if (!mDbConn->getQuality(quality))
		return;

	mNewInp->setQuality(quality);
}

void CivBaseInpBuilder::buildSources()
{
	ASSERT(mDbConn, mNewInp)

		CivSources sources;
	if (!mDbConn->getSources(sources))
		return;

	mNewInp->setSources(sources);
}

void CivBaseInpBuilder::buildMixing()
{
	ASSERT(mDbConn, mNewInp)

		CivMixing mixing;
	if (!mDbConn->getMixing(mixing))
		return;

	mNewInp->setMixing(mixing);
}

void CivBaseInpBuilder::buildCurves()
{
	ASSERT(mDbConn, mNewInp)

		/*CivCurves curves;
		if (!mDbConn->getCurves(curves))
			return;

		mNewInp->setCurves(curves);*/
}

void CivBaseInpBuilder::buildStatus()
{
	ASSERT(mDbConn, mNewInp)

		CivStatus status;
	if (!mDbConn->getStatus(status))
		return;

	mNewInp->setStatus(status);
}

void CivBaseInpBuilder::buildControls()
{

}

void CivBaseInpBuilder::buildRules()
{

}

void CivBaseInpBuilder::buildDemands()
{
	ASSERT(mDbConn, mNewInp)

		CivDemands mand;
	if (!mDbConn->getDemands(mand))
		return;

	mNewInp->setDemands(mand);
}