#include "CivTrackingAnalysis.h"
#include "CivSimulResultCache.h"
#include "CivHydrSimulation.h"
#include "CivGraphFactory.h"
#include "CivDbConn.h"

CivTrackingAnalysis::CivTrackingAnalysis()
	:mGraph(new ALGraph<CivGraphJunction, CivGraphEdage>())
{

}

CivTrackingAnalysis::~CivTrackingAnalysis()
{
	delete mGraph;
}

bool CivTrackingAnalysis::createGraphFrom(CivDbConn* dbConn)
{
	if (dbConn == nullptr)
		return false;

	CivHydrSimulation simulation;
	simulation.registDb(dbConn);

	if (!simulation.hdyrInstantSimulation())
		return false;

	NodeResultItems nodeItems;
	LinkResultItems linkItems;
	simulation.getDataByTime(0, nodeItems, linkItems);
	if (nodeItems.size() <= 0)
		return false;

	// 
	CivCoordinates coords;
	dbConn->getCoordinates(coords);

	std::list<CivCoordinates::CoordTable> coordTableLis = coords.mTables;
	std::list<CivCoordinates::CoordTable>::iterator iter;
	for (iter = coordTableLis.begin(); iter != coordTableLis.end(); iter++)
	{
		CivCoordinates::CoordTable coordTable = *iter;

		// 붥
		CivGraphJunction graphJunction(coordTable.ID, coordTable.XCoord, coordTable.YCoord);
		auto iter = nodeItems.find(coordTable.ID);
		if (iter != nodeItems.end())
		{
			graphJunction.setHead(iter->second.dHead);
		}
		mGraph->insertAVertex(graphJunction);
	}

	// ܶ
	CivPipe civPipe;
	dbConn->getPipe(civPipe);

	std::list<CivPipe::PipesTable> pipesTableLis = civPipe.mTables;
	std::list<CivPipe::PipesTable>::iterator pIter;

	for (pIter = pipesTableLis.begin(); pIter != pipesTableLis.end(); pIter++)
	{
		CivPipe::PipesTable pipeTable = *pIter;

		// ȡܶ
		CivGraphJunction graphJunction1(pipeTable.Node1);
		int index1 = mGraph->getVertexIndex(graphJunction1);
		CivGraphJunction sStartjuction = mGraph->getData(index1);

		// ȡܶյֵ
		CivGraphJunction graphJunction2(pipeTable.Node2);
		int index2 = mGraph->getVertexIndex(graphJunction2);
		CivGraphJunction endJunction = mGraph->getData(index2);


		if (sStartjuction.getHead() > endJunction.getHead())
		{
			CivGraphEdage edAge(pipeTable.ID, pipeTable.Length, { sStartjuction.getXCoord(),sStartjuction.getYCoord(),endJunction.getXCoord(),endJunction.getYCoord() });
			mGraph->insertAEdge(sStartjuction, endJunction, edAge);
		}
		else if (sStartjuction.getHead() == endJunction.getHead())
		{
			CivGraphEdage edAge1(pipeTable.ID, pipeTable.Length, { sStartjuction.getXCoord(),sStartjuction.getYCoord(),endJunction.getXCoord(),endJunction.getYCoord() });
			mGraph->insertAEdge(sStartjuction, endJunction, edAge1);

			CivGraphEdage edAge2(pipeTable.ID, pipeTable.Length, { endJunction.getXCoord(),endJunction.getYCoord(), sStartjuction.getXCoord(),sStartjuction.getYCoord() });
			mGraph->insertAEdge(endJunction, sStartjuction, edAge2);

		}
		else
		{
			CivGraphEdage edAge(pipeTable.ID, pipeTable.Length, { endJunction.getXCoord(),endJunction.getYCoord(), sStartjuction.getXCoord(),sStartjuction.getYCoord() });
			mGraph->insertAEdge(endJunction, sStartjuction, edAge);
		}

	}
	return true;
}

bool CivTrackingAnalysis::transformJson(const std::vector<CivGraphJunction>& junctions,
	const std::vector<CivGraphEdage>& pipes, std::string& jsonResult)
{

	jsonResult.append("[");

	//// ڵ
	//size_t juncTotal = junctions.size();
	//for (int i = 0; i < juncTotal; i++)
	//{
	//	CivGraphJunction junction = junctions.at(i);

	//	std::string sn = junction.getSn();
	//	std::string xCoord = junction.getXCoord();
	//	std::string yCoord = junction.getYCoord();

	//	jsonResult.append("[");
	//	jsonResult.append(sn);
	//	jsonResult.append(",");
	//	jsonResult.append(xCoord);
	//	jsonResult.append(",");
	//	jsonResult.append(yCoord);
	//	jsonResult.append("],");
	//}
	//jsonResult = jsonResult.substr(0, jsonResult.length() - 1);
	//jsonResult.append("],\"pipes\":[");

	// ܶ
	size_t pipesTotal = pipes.size();
	for (int i = 0; i < pipesTotal; i++)
	{
		CivGraphEdage edge = pipes[i];

		if (edge.getSN().empty())
			continue;

		jsonResult.append("[[");
		jsonResult.append(edge.getStartX());
		jsonResult.append(",");
		jsonResult.append(edge.getStartY());
		jsonResult.append("],[");
		jsonResult.append(edge.getEndX());
		jsonResult.append(",");
		jsonResult.append(edge.getEndY());
		jsonResult.append("]],");
	}
	jsonResult = jsonResult.substr(0, jsonResult.length() - 1);
	jsonResult.append("]");

	return true;
}

bool CivTrackingAnalysis::upstreamTracking(const std::string& sN, std::string& jsonResult)
{	
	std::vector<CivGraphJunction> junctions;
	std::vector<CivGraphEdage> edges;
	
	if (!upstreamTracking(sN, junctions, edges))
	{
		return false;
	}

	// ѯϢƴӳַ
	if (!transformJson(junctions, edges, jsonResult))
	{
		return false;
	}

	return true;
}

bool CivTrackingAnalysis::downstreamTracking(const std::string& sN, std::string& jsonResult)
{
	std::vector<CivGraphJunction> junctions;
	std::vector<CivGraphEdage> edges;

	if (!downstreamTracking(sN, junctions, edges))
	{
		return false;
	}

	// ѯϢƴӳַ
	if (!transformJson(junctions, edges,jsonResult))
	{
		return false;
	}

	return true;
}

bool CivTrackingAnalysis::waterSupplyScopeAnalysis(const std::string& sN, std::string& jsonResult)
{

	return true;
}

bool CivTrackingAnalysis::upstreamTracking(const std::string& sN,
	std::vector<CivGraphJunction>& junctions,
	std::vector<CivGraphEdage>& pipes)
{
	// ݣͼ
	ALGraph<CivGraphJunction, CivGraphEdage>* graph = mGraph->reverseGraph();
	
	CivGraphJunction junction(sN);
	graph->getVisitedResult(junction, junctions, pipes);

	delete graph;
	return true;
}

bool CivTrackingAnalysis::downstreamTracking(const std::string& sN,
	std::vector<CivGraphJunction>& junctions,
	std::vector<CivGraphEdage>& pipes)
{

	CivGraphJunction junction(sN);
	mGraph->getVisitedResult(junction, junctions, pipes);

	return true;
}

bool CivTrackingAnalysis::waterSupplyScopeAnalysis(const std::string& sN,
	std::vector<CivGraphJunction>& junctions,
	std::vector<CivGraphEdage>& pipes)
{

	return true;
}