#include "CivPgConn.h"
#include "CivCommonUtils.h"
#include "CivSysLog.h"
#include "CivTypes.h"
#include <sstream>


CivPgConn::CivPgConn(char* uri)
	: CivDbConn(),mUri(uri)
{
	mConn = NULL;
	mResult = NULL;
}

CivPgConn::~CivPgConn()
{
    if (mConn != nullptr)
    {
        PQfinish(mConn);
    }
}

bool CivPgConn::deleteByField(StrQuote table, StrQuote filedName, StrQuote fieldValue)
{
    std::string delSql = "delete from public.\"";
    delSql.append(table);
    delSql.append("\" where   ");
    delSql.append(filedName);
    delSql.append("='");
    delSql.append(fieldValue);
    delSql.append("'");

    std::string utf8Sql = TransUTFCoding(delSql);

    if (!execSql(utf8Sql))
    {
        CivSysLog::getInstance()->error(mLastError, "CivPgConn", __FUNCTION__);
        return false;
    }
        

    return true;
}

bool CivPgConn::updateNode(const NodeResultItems& nodeItems)
{
    if (!isValid())
        return false;

    std::stringstream os;
    NodeFieldName filedNames;

    os << "INSERT INTO public." << NODERESULTTABLE << " ( ";
    os << filedNames.Number << ",";
    os << filedNames.dDemand << ",";
    os << filedNames.dHead << ",";
    os << filedNames.dElevation << ",";
    os << filedNames.dPressure << ",";
    os << filedNames.dTankLevel << ",";
    os << filedNames.dTankMaxVolume << ",";
    os << filedNames.dTankVolume << ",";
    os << filedNames.dDate << ",";
    os << filedNames.dInterval << ",";
    os << filedNames.dQuality << ",";
    os << filedNames.dModifyTime;
    os << ") VALUES";

    std::string preSql = os.str();
    os.flush();
    os.clear();
    os.str("");

    // 2, ֵ
    for (auto iter = nodeItems.begin(); iter != nodeItems.end(); iter++)
    {
        NodeResultItem item = iter->second;
        os << "('" << item.szNo << "',";
        os << item.dDemand << ",";
        os << item.dHead << ",";
        os << item.dElevation << ",";
        os << item.dPressure << ",";
        os << item.dTankLevel << ",";
        os << item.dTankMaxVolume << ",";
        os << item.dTankVolume << ",";
        os << "'" << CurrentDate << "',";
        os << item.dInterval << ",";
        os << item.dQuality << ",";
        os << "'" << CurrentTime << "'";

        auto subiter = iter;
        ++subiter;
        if (subiter == nodeItems.end())
        {
            os << ")";
        }
        else
        {
            os << "),";
        }
    }
    std::string sql = TransUTFCoding(preSql) + os.str();
    os.clear();
    os.str("");
    if (!execSql(sql))
    {
        CivSysLog::getInstance()->error(mLastError, "CivPgConn", __FUNCTION__);
        return false;
    }
    return true;
}

bool CivPgConn::updateLink(const LinkResultItems& lineItems)
{
    if (!isValid())
        return false;

    std::stringstream os;
    LinkFiledName filedNames;
    std::string currData = CurrentDate;

    os << "INSERT INTO public." << PIPERESULTTABLE << "( ";
    os << filedNames.szNo << ",";
    os << filedNames.dFlow << ",";
    os << filedNames.nFlowDirect << ",";
    os << filedNames.dVelocity << ",";
    os << filedNames.dHeadloss << ",";
    os << filedNames.dUnitHeadloss << ",";
    os << filedNames.dFromNodHeadloss << ",";
    os << filedNames.dToNodHeadloss << ",";
    os << filedNames.dLocalHeadloss << ",";
    os << filedNames.dFrictionHeadloss << ",";
    os << filedNames.szStatus << ",";
    os << filedNames.dDate << ",";
    os << filedNames.dInterval << ",";
    os << filedNames.dQuality << ",";
    os << filedNames.dModifyTime;
    os << ") VALUES";

    std::string preSql = os.str();
    os.clear();
    os.str("");

    // 2, ֵ
    for (auto iter = lineItems.begin(); iter != lineItems.end(); iter++)
    {
        LinkResultItem item = iter->second;
        os << "('" << item.szNo << "'" << ",";
        os << item.dFlow << ",";
        os << item.nFlowDirect << ",";
        os << item.dVelocity << ",";
        os << item.dHeadloss << ",";
        os << item.dUnitHeadloss << ",";
        os << item.dFromNodHeadloss << ",";
        os << item.dToNodHeadloss << ",";
        os << item.dLocalHeadloss << ",";
        os << item.dFrictionHeadloss << ",";
        os << "'" << TransUTFCoding(item.szStatus) << "'" << ",";
        os << "'" << CurrentDate << "',";
        os << item.dInterval << ",";
        os << item.dQuality << ",";
        os << "'" << CurrentTime << "'";

        auto subIte = iter;
        ++subIte;
        if (subIte == lineItems.end())
            os << ")";
        else
            os << "),";
    }
    std::string utf8Sql = TransUTFCoding(preSql);
    std::string sql = utf8Sql + os.str();

    os.clear();
    os.str("");

    if (!execSql(sql))
    {
        CivSysLog::getInstance()->error(mLastError, "CivPgConn", __FUNCTION__);
        return false;
    }
    return true;
}

bool CivPgConn::createTable(CivTableTemp& temp)
{
    if (&temp == NULL)
        return false;

    Str sql = temp.createSql();
    Str finalSql = TransUTFCoding(sql);

    return execSql(finalSql);
}

bool CivPgConn::open()
{
    if (mIsOpen)
        return true;

    mConn = PQconnectdb(mUri);
    if (PQstatus(mConn) == CONNECTION_BAD) {
        mConn = NULL;
        PQfinish(mConn);
        mIsOpen = false;
        return false;
    }
    mIsOpen = true;
    return true;
}

void CivPgConn::close()
{
    if (mConn != NULL)
    {
        PQfinish(mConn);
    }       
    mIsOpen = false;
}

bool CivPgConn::tableExist(StrQuote tableName)
{
    char sql[256] = "select count(*) from pg_class where relname = '";
    strcat_s(sql, tableName.c_str());
    strcat_s(sql, "'");

    if (!execSql(sql))
    {
        CivSysLog::getInstance()->error(mLastError, "CivPgConn", __FUNCTION__);
        return false;
    }
    char* res = PQgetvalue(mResult, 0, 0);
    PQclear(mResult);
    int number = atoi(res);

    return number > 0 ? true : false;
}

bool CivPgConn::execSql(StrQuote sql)
{
    if (sql.empty())
        return false;

    if (!isValid())
        open();
  
    mResult = PQexec(mConn, sql.c_str());
    char* error = PQresultErrorMessage(mResult);
    std::string errorStr = error;

    if (!errorStr.empty())
    {
        mLastError = error;
        PQclear(mResult);
        return false;
    }
    return true;
}

bool CivPgConn::getNode(CivNode& node)
{
    
    CivJunctionTableTemp::JunctionTable nodeTable;

    char sql[256] = "select ";
    strcat_s(sql, nodeTable.thePoint.c_str());
    strcat_s(sql, ",");
    strcat_s(sql, nodeTable.elev.c_str());
    strcat_s(sql, ",");
    strcat_s(sql, nodeTable.demand.c_str());
    strcat_s(sql, ",");
    strcat_s(sql, nodeTable.pattern.c_str());
    strcat_s(sql, " from public.\"");
    strcat_s(sql, PIPENODE.c_str());
    strcat_s(sql, "\"");

    std::string utf8Sql = TransUTFCoding(sql);
    if (!execSql(utf8Sql))
    {
        CivSysLog::getInstance()->error(mLastError, "CivPgConn", __FUNCTION__);
        return false;
    }

    int rows = PQntuples(mResult);

    for (int i = 0; i < rows; i++)
    {
        CivNode::NodeTable nodeTable;
        nodeTable.ID = PQgetvalue(mResult, i, 0);
        nodeTable.Elev = PQgetvalue(mResult, i, 1);
        nodeTable.Demand = PQgetvalue(mResult, i, 2);
        nodeTable.PatternId = PQgetvalue(mResult, i, 3);
        node.addItem(nodeTable);
    }
    PQclear(mResult);
    return true;
}

bool CivPgConn::getPipe(CivPipe& pipe)
{
    CivPipeTableTemp::PipeTable pipeTable;

    //;ID              	Node1           	Node2           	Length      	Diameter    	Roughness   	MinorLoss   	Status
    std::string sql = "select ";
    sql.append(pipeTable.snNo.c_str());
    sql.append(",");
    sql.append(pipeTable.lastPoint);
    sql.append(",");
    sql.append(pipeTable.thePoint);
    sql.append(",");
    sql.append(pipeTable.length);
    sql.append(",");
    sql.append(pipeTable.diameter);
    sql.append(",");
    sql.append(pipeTable.friction);
    sql.append(",");
    sql.append(pipeTable.localHeadLoss);
    sql.append(",");
    sql.append(pipeTable.status);
    sql.append(" from  ");
    sql.append(PIPELINE);

    std::string utf8Sql = TransUTFCoding(sql);
    if (!execSql(utf8Sql))
    {
        CivSysLog::getInstance()->error(mLastError, "CivPgConn", __FUNCTION__);
        return false;
    }

    int rows = PQntuples(mResult);
    int columns = PQnfields(mResult);
    for (int i = 0; i < rows; i++)
    {
        CivPipe::PipesTable pipeTable;
        pipeTable.ID = PQgetvalue(mResult, i, 0);
        pipeTable.Node1 = PQgetvalue(mResult, i, 1);
        pipeTable.Node2 = PQgetvalue(mResult, i, 2);
        pipeTable.Length = PQgetvalue(mResult, i, 3);
        pipeTable.Diameter = PQgetvalue(mResult, i, 4);
        pipeTable.Roughness = PQgetvalue(mResult, i, 5);
        pipeTable.MinorLoss = PQgetvalue(mResult, i, 6);
        pipeTable.Status = PQgetvalue(mResult, i, 7);

        pipe.addItem(pipeTable);
    }
    PQclear(mResult);
    return true;
}

bool CivPgConn::getTank(CivTank& tank)
{
    CivTankTableTemp::TankTable tankTable;

    //;ID              	Elevation   	InitLevel   	MinLevel    	MaxLevel    	Diameter    	MinVol      	VolCurve
    std::string sql = "select ";
    sql.append(tankTable.thePoint);
    sql.append(",");
    sql.append(tankTable.elev);
    sql.append(",");
    sql.append(tankTable.initLevel);
    sql.append(",");
    sql.append(tankTable.lowLevel);
    sql.append(",");
    sql.append(tankTable.highLevel);
    sql.append(",");
    sql.append(tankTable.diametor);
    sql.append(",");
    sql.append(tankTable.capacity);
    sql.append(",");
    sql.append(tankTable.volCurve);
    sql.append(" from  public.\"");
    sql.append(TANK);
    sql.append("\"");

    std::string utf8Sql = TransUTFCoding(sql);
    if (!execSql(utf8Sql))
    {
        CivSysLog::getInstance()->error(mLastError, "CivPgConn", __FUNCTION__);
        return false;
    }

    int rows = PQntuples(mResult);
    int columns = PQnfields(mResult);
    for (int i = 0; i < rows; i++)
    {
        CivTank::TankTable tankTable;
        tankTable.ID= PQgetvalue(mResult, i, 0);
        tankTable.Elev = PQgetvalue(mResult, i, 1);
        tankTable.InitLevel= PQgetvalue(mResult, i, 2);
        tankTable.MinLevel= PQgetvalue(mResult, i, 3);
        tankTable.MaxLevel= PQgetvalue(mResult, i, 4);
        tankTable.Diameter= PQgetvalue(mResult, i, 5);
        tankTable.MinVol= PQgetvalue(mResult, i, 6);
        tankTable.VolCurve= PQgetvalue(mResult, i, 7);
     
        tank.addItem(tankTable);
    }
    PQclear(mResult);
    return true;
}

bool CivPgConn::getValve(CivValve& valve)
{

    CivValveTableTemp::ValveTable vaveTable;

    //;ID              	Node1           	Node2           	Length      	Diameter    	Roughness   	MinorLoss   	Status
    std::string sql = "select ";
    sql.append(vaveTable.id);
    sql.append(",");
    sql.append(vaveTable.startPoint);
    sql.append(",");
    sql.append(vaveTable.endPoint);
    sql.append(",");
    sql.append(vaveTable.diameter);
    sql.append(",");
    sql.append(vaveTable.type);
    sql.append(",");
    sql.append(vaveTable.setting);
    sql.append(",");
    sql.append(vaveTable.loss);
    sql.append(" from  public.\"");
    sql.append(VALVE);
    sql.append("\"");

    std::string utf8Sql = TransUTFCoding(sql);
    if (!execSql(utf8Sql))
    {
        CivSysLog::getInstance()->error(mLastError, "CivPgConn", __FUNCTION__);
        return false;
    }

    int rows = PQntuples(mResult);
    int columns = PQnfields(mResult);
    for (int i = 0; i < rows; i++)
    {
        CivValve::ValveTable vaveTable;
        vaveTable.ID = PQgetvalue(mResult, i, 0);
        vaveTable.Node1 = PQgetvalue(mResult, i, 1);
        vaveTable.Node2 = PQgetvalue(mResult, i, 2);
        vaveTable.Diameter= PQgetvalue(mResult, i, 3);
        vaveTable.Type = PQgetvalue(mResult, i, 4);
        vaveTable.Setting = PQgetvalue(mResult, i, 5);
        vaveTable.MinorLoss = PQgetvalue(mResult, i, 6);
    
        valve.addItem(vaveTable);
    }
    PQclear(mResult);
    return true;
}

bool CivPgConn::getPumps(CivPumps& pump)
{

    CivPumpTableTemp::PumpTable pmTable;

    //;ID              	Node1           	Node2           	Length      	Diameter    	Roughness   	MinorLoss   	Status
    std::string sql = "select ";
    sql.append(pmTable.thePoint);
    sql.append(",");
    sql.append(pmTable.startPoint);
    sql.append(",");
    sql.append(pmTable.endPoint);
    sql.append(",");
    sql.append(pmTable.headCurve);
    sql.append(" from  public.\"");
    sql.append(PUMP);
    sql.append("\"");

    std::string utf8Sql = TransUTFCoding(sql);
    if (!execSql(utf8Sql))
    {
        return false;
    }

    int rows = PQntuples(mResult);
    int columns = PQnfields(mResult);
    for (int i = 0; i < rows; i++)
    {
        CivPumps::PumpTable pumpTable;
        pumpTable.ID = PQgetvalue(mResult, i, 0);
        pumpTable.Node1 = PQgetvalue(mResult, i, 1);
        pumpTable.Node2 = PQgetvalue(mResult, i, 2);
        pumpTable.Parameters = PQgetvalue(mResult, i, 3);
      
        pump.addItem(pumpTable);
    }
    PQclear(mResult);
    return true;
}

bool CivPgConn::getReservoirs(CivReservoirs& reservoirs)
{
    ResourcesTable restemTable;

    //;ID              	Head        	Pattern     
    std::string sql = "select ";
    sql.append(restemTable.sn);
    sql.append(",");
    sql.append(restemTable.head);
    sql.append(",");
    sql.append(restemTable.headPattern);
    sql.append(" from  public.\"");
    sql.append(RESIVOIR);
    sql.append("\"");

    std::string utf8Sql = TransUTFCoding(sql);
    if (!execSql(utf8Sql))
    {
        CivSysLog::getInstance()->error(mLastError, "CivPgConn", __FUNCTION__);
        return false;
    }

    int rows = PQntuples(mResult);
    for (int i = 0; i < rows; i++)
    {
        CivReservoirs::ReservoirsTable resTable;
        resTable.ID = PQgetvalue(mResult, i, 0);
        resTable.Head = PQgetvalue(mResult, i, 1);
        resTable.Pattern = PQgetvalue(mResult, i, 2);
     
        reservoirs.addItem(resTable);
    }
    PQclear(mResult);
    return true;
}

bool CivPgConn::getEmitters(CivEmitters& emitter)
{
    return true;
}

bool CivPgConn::getCoordinates(CivCoordinates& coord)
{
    CivJunctionTableTemp::JunctionTable nodeTable;

    std::string sql = "select ";
    sql.append(TransUTFCoding(nodeTable.thePoint));
    sql.append(",");
    sql.append(TransUTFCoding(nodeTable.xCoord));
    sql.append(",");
    sql.append(TransUTFCoding(nodeTable.yCoord));
    sql.append(" from public.\"");
    sql.append(TransUTFCoding(PIPENODE));
    sql.append("\"");

    sql.append(" UNION  select ");
    sql.append(TransUTFCoding(nodeTable.thePoint));
    sql.append(",");
    sql.append(TransUTFCoding(nodeTable.xCoord));
    sql.append(",");
    sql.append(TransUTFCoding(nodeTable.yCoord));
    sql.append(" from public.\"");
    sql.append(TransUTFCoding(TANK));
    sql.append("\"");

    sql.append(" UNION  select ");
    sql.append(TransUTFCoding(nodeTable.thePoint));
    sql.append(",");
    sql.append(TransUTFCoding(nodeTable.xCoord));
    sql.append(",");
    sql.append(TransUTFCoding(nodeTable.yCoord));
    sql.append(" from public.\"");
    sql.append(TransUTFCoding(RESIVOIR));
    sql.append("\"");

    if (!execSql(sql))
    {
        return false;
    }

    int rows = PQntuples(mResult);
    for (int i = 0; i < rows; i++)
    {
        CivCoordinates::CoordTable coordTable;
        coordTable.ID = PQgetvalue(mResult, i, 0);
        coordTable.XCoord = PQgetvalue(mResult, i, 1);
        coordTable.YCoord = PQgetvalue(mResult, i, 2);
        
        coord.addItem(coordTable);
    }
    PQclear(mResult);
    return true;
}

bool CivPgConn::getMixing(CivMixing& mixing)
{
    return true;
}

bool CivPgConn::getParameter(std::vector<CivParameter>& params)
{
    CivParametorTableTemp::ParametorTable restemTable;

    std::string sql = "select ";
    sql.append(restemTable.name);
    sql.append(",");
    sql.append(restemTable.val);
    sql.append(",");
    sql.append(restemTable.type);
    sql.append(" from  public.\"");
    sql.append(OPTIONTTABLE);
    sql.append("\"");

    std::string utf8Sql = TransUTFCoding(sql);
    if (!execSql(utf8Sql))
    {
        CivSysLog::getInstance()->error(mLastError, "CivPgConn", __FUNCTION__);
        return false;
    }

    int rows = PQntuples(mResult);
 
    std::map<std::string, std::vector<CivParameter::ParamTable>> tempMap;
    CivParameter::ParamTable resTable;
    for (int i = 0; i < rows; i++)
    {
        CivParameter::ParamTable pTable;
        pTable.name = PQgetvalue(mResult, i, 0);
        pTable.val = PQgetvalue(mResult, i, 1);

        char* type = PQgetvalue(mResult, i, 2);
        tempMap[type].push_back(pTable);
    }
    // ʱ洢ƴӲ
    for (auto iter = tempMap.begin(); iter != tempMap.end(); iter++)
    {
        CivParameter param;
        std::string type = iter->first;
        std::vector<CivParameter::ParamTable> paramTables = iter->second;

        param.setType(type);

        int total = paramTables.size();
        for (int i = 0; i < total; i++)
            param.addItem(paramTables[i]);
       
        params.push_back(param);
    }
    PQclear(mResult);

    return true;
}

bool CivPgConn::getPatterns(CivPatterns& patterns)
{
    CivPatternTableTemp::PatternTable patternTable;

    std::string sql= "select ";
    sql.append(patternTable.modelName);
    sql.append(",");
    sql.append(patternTable.modelData);
    sql.append(",");
    sql.append(patternTable.modelType);
    sql.append(" from  public.\"");
    sql.append(PATTERNTABLE);
    sql.append("\"");

    std::string utf8Sql = TransUTFCoding(sql);
    if (!execSql(utf8Sql))
    {
        CivSysLog::getInstance()->error(mLastError, "CivPgConn", __FUNCTION__);
        return false;
    }

    int rows = PQntuples(mResult);
  
    for (int i = 0; i < rows; i++)
    {     
       char* name  = PQgetvalue(mResult, i, 0);
       char* val = PQgetvalue(mResult, i, 1);
      
       CivPatterns::PatternTable pTable;
       pTable.name = name;
       pTable.val = val;

       patterns.addItem(pTable);
    }

    PQclear(mResult);

    return true;
}

bool CivPgConn::getCurves(CivCurves& curves)
{
    CivCurveTableTemp::CurveTable curveTable;

    std::string sql = "select ";
    sql.append(curveTable.CurveName);
    sql.append(",");
    sql.append(curveTable.CurveType);
    sql.append(",");
    sql.append(curveTable.CurveData);
    sql.append(" from  public.\"");
    sql.append(CURVETABLE);
    sql.append("\"");

    std::string utf8Sql = TransUTFCoding(sql);
    if (!execSql(utf8Sql))
    {
        CivSysLog::getInstance()->error(mLastError, "CivPgConn", __FUNCTION__);
        return false;
    }

    int rows = PQntuples(mResult);
    for (int i = 0; i < rows; i++)
    {
        CivCurves::CurvesTable cTable;
        cTable.ID = PQgetvalue(mResult, i, 0);
        cTable.XCoord = PQgetvalue(mResult, i, 1);
        cTable.YCoord =  PQgetvalue(mResult, i,2);
        curves.addItem(cTable);
    }
    PQclear(mResult);
    return true;
}   

bool CivPgConn::getDemands(CivDemands& demands)
{
    return true;
}

bool CivPgConn::getSources(CivSources& sources)
{
    return true;
}

bool CivPgConn::getStatus(CivStatus& status)
{
    return true;
}

bool CivPgConn::getQuality(CivQuality& quality)
{
    CivJunctionTableTemp::JunctionTable nodeTable;

    std::string sql = "select ";
    sql.append(nodeTable.thePoint);
    sql.append(",");
    sql.append(nodeTable.initQuality);
    sql.append(" from ");
    sql.append(PIPENODE);

    std::string utf8Sql = TransUTFCoding(sql);
    if (!execSql(utf8Sql))
    {
        CivSysLog::getInstance()->error(mLastError, "CivPgConn", __FUNCTION__);
        return false;
    }

    int rows = PQntuples(mResult);
    int columns = PQnfields(mResult);
    for (int i = 0; i < rows; i++)
    {
        CivQuality::QualityTable quliTable;
        quliTable.ID = PQgetvalue(mResult, i, 0);
        quliTable.InitQuality = PQgetvalue(mResult, i, 1);
   
        quality.addItem(quliTable);
    }
    PQclear(mResult);
    return true;
}

bool CivPgConn::getLabels(CivLabels& labels)
{
    return true;
}

bool CivPgConn::getTags(CivTags& tags)
{
    return true;
}

bool CivPgConn::getFieldValueFromTable(
    const std::string& table,
    const std::vector<std::string>& fileds, 
    std::vector< std::vector<std::string>>& result)
{
    std::string sql;
    sql.append("select ");

    size_t total = fileds.size();
    for (int i = 0; i < total; i++)
    {
        sql.append(fileds[i]);

        if (i < (total - 1))
            sql.append(",");      
    }
    sql.append(" from public.\"");
    sql.append(table);
    sql.append("\"");

    std::string utf8Sql = TransUTFCoding(sql);
    if (!execSql(utf8Sql))
    {
        CivSysLog::getInstance()->error(mLastError, "CivPgConn", __FUNCTION__);
        return false;
    }

    int rows = PQntuples(mResult);
    int columns = PQnfields(mResult);

    for (int i = 0; i < rows; i++)
    {
        std::vector<std::string> vecRes;
        for (int j = 0; j < columns; j++)
        {
            std::string res  = PQgetvalue(mResult, i, j);
            vecRes.push_back(res);
        }  
        result.push_back(vecRes);
    }

    PQclear(mResult);

    return true;
}

bool CivPgConn::getValuesFromTableByCondition(
    const std::string& table,
    const std::string& condition,
    const std::vector<std::string>& fileds,
    std::vector< std::map<std::string,std::string>>& result)
{

    std::string sql;
    sql.append("select ");

    size_t total = fileds.size();
    for (int i = 0; i < total; i++)
    {
        sql.append(fileds[i]);
        sql.append(",");      
    }
    sql = sql.substr(0, sql.length() - 1);
    sql.append(" from public.\"");
    sql.append(table);
    sql.append("\"  where ");
    sql.append(condition);

    std::string utf8Sql = TransUTFCoding(sql);
    if (!execSql(utf8Sql))
    {
        CivSysLog::getInstance()->error(mLastError, "CivPgConn", __FUNCTION__);
        return false;
    }

    int rows = PQntuples(mResult);
    int columns = PQnfields(mResult);

    for (int i = 0; i < rows; i++)
    {
        std::map<std::string, std::string> tempMap;
        for (int j = 0; j < columns; j++)
        {
            std::string res = PQgetvalue(mResult, i, j);
            std::string filedName = UTFTransCoding(PQfname(mResult, j));

            tempMap.insert(std::pair<std::string, std::string>(filedName,res));   
        }
        result.push_back(tempMap);
    }
    PQclear(mResult);

    return true;
}